// ModelSwarm.m		 - adapted from SimpleBug app

#import "ModelSwarm.h"
#import <random.h>
#import <activity.h>
#import <collections.h>

@implementation ModelSwarm  

// These methods provide access to the objects inside the ModelSwarm.
// These objects are the ones visible to other classes via message call.
// In theory we could just let other objects use Probes to read our state,
// but message access is frequently more convenient.

- getBugList
{
  return bugList;
}

- getMaster
{
  return masterList;
}

- getWorld
{
  return world;
}

- getObjs
{
  return objs;
}

+ createBegin: aZone 
{
  ModelSwarm *obj;
  id <ProbeMap> probeMap;

  // in createBegin, we set up the simulation parameters

  // First, call our superclass createBegin - the return value is the
  // allocated BugSwarm object.

  obj = [super createBegin: aZone];

  // Now fill in various simulation parameters with default values.

  obj->worldXSize = 80;
  obj->worldYSize = 80;
  obj->seedProbObst = 0.01;
  obj->seedProbFood = 0.005;
  obj->bugDensity = 0.01;
  obj->bugLifeTime = 400;
  obj->DELTA = 0.3;
  obj->delta = 0.3;

  // And build a customized probe map. Without a probe map, the default
  // is to show all variables and messages. Here we choose to
  // customize the appearance of the probe, give a nicer interface.

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  // Add in a bunch of variables, one per simulation parameter

  [probeMap addProbe: [probeLibrary getProbeForVariable: "worldXSize"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "worldYSize"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "seedProbObst"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "seedProbFood"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "bugDensity"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "bugLifeTime"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "DELTA"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "delta"
                                    inClass: [self class]]];

  // Now install our custom probeMap into the probeLibrary.

  [probeLibrary setProbeMap: probeMap For: [self class]];
  
  // We've created the BugSwarm and initialized it.
  // return the id of the newly created Swarm

  return obj;
}

- createEnd
{
  return [super createEnd];
}

- buildObjects
{
  Bug *aBug;
  Master *master;
  NavDest *navDest;
  int x,y;
  
  // Here, we create the objects in the model

  // Then, create the objects space and initialize it

  objs = [ObjsSpace createBegin: self];
  [objs setSizeX: worldXSize Y: worldYSize];
  objs = [objs createEnd];
  
  [objs seedObjsWithProb: seedProbObst Food:seedProbFood];

  // Now set up the grid used to represent agent position
  // Grid2d enforces only 1 bug per site

  world = [Grid2d createBegin: self];
  [world setSizeX: worldXSize Y: worldYSize];
  world = [world createEnd];
  [world fillWithObject: nil];

  // create some slaves

  bugList = [List create: self];  // a list of bugs
  navList = [List create: self];  // what the master has to remember for each bug

  x = ([uniformDblRand getDoubleWithMin: 0.0 withMax: 1.0]) * (worldXSize-1);
  y = ([uniformDblRand getDoubleWithMin: 0.0 withMax: 1.0]) * (worldYSize-1);
  for (y = 0; y < worldYSize; y++)
    for (x = 0; x < worldXSize; x++) 
      if ([uniformDblRand getDoubleWithMin: 0.0 withMax: 1.0] < bugDensity)
        {
          aBug = [Bug createBegin: self];
          [aBug setWorld: world Objs: objs];
          aBug = [aBug createEnd];
          [aBug setX: x Y: y];
          [bugList addLast: aBug];
          [aBug initWeights];  // set random weights in NN
          [aBug setLifeTime: bugLifeTime];
          [aBug setLearningConstants: DELTA: delta];

          navDest = [NavDest createBegin: self];
          navDest = [navDest createEnd];
          [navList addLast: navDest];
        }

  // create master
  
  x = (worldXSize/2)-1;  // place in the middle of the screen 
  y = (worldYSize/2)-1;
  master = [Master createBegin: self];
  [master setWorld: world Objs: objs];
  [master setBugList: bugList];
  master = [master createEnd];

  [master setNavList: navList];  // set initial destinations for each bug and save in navList
  [master setX: x Y: y];

  masterList = [List create: self];  // use list for more simplicity
  [masterList addLast: master];

  return self;
}

- buildActions
{
  // Create the list of simulation actions. We put these in an action
  // group, because we want these actions to be executed in a specific
  // order, but these steps should take no (simulated) time. The
  // M(foo) means "The message called <foo>". You can send a message
  // To a particular object, or ForEach object in a collection.

  modelActions = [ActionGroup create: self];
  [modelActions createActionForEach: masterList message: M(speak)];

  // Then we create a schedule that executes the modelActions. modelActions
  // is an ActionGroup, by itself it has no notion of time. In order to
  // have it executed in time, we create a Schedule that says to use
  // the modelActions ActionGroup at particular times.
  // This schedule has a repeat interval of 1, it will loop every time step.
  // The action is executed at time 0 relative to the beginning of the loop.

  // This is a simple schedule, with only one action that is just
  // repeated every time. See mousetraps for more complicated schedules.

  modelSchedule = [Schedule createBegin: self];
  [modelSchedule setRepeatInterval: 1];
  modelSchedule = [modelSchedule createEnd];
  [modelSchedule at: 0 createAction: modelActions]; 

  return self;
}

- activateIn: swarmContext
{
  // Here, we activate the swarm in the context passed in
  // Then we activate our schedule in ourselves

  [super activateIn: swarmContext];

  [modelSchedule activateIn: self];

  return [self getSwarmActivity];
}

@end



  


